import json
import logging

from werkzeug.wrappers import Request, Response
from werkzeug.exceptions import HTTPException, abort

import alexandra.util as util


log = logging.getLogger(__name__)


class WsgiApp:
    """This class is compatible with Werkzeug's WSGI implementation and
    handles all the gory details of parsing Alexa requests and validating that
    they were actually sent by Amazon.
    """

    def __init__(self, alexa, validate_requests=True):
        """
        :param alexa: alexandra.app.App to wrap
        :param validate_requests: Whether or not to do timestamp and
            certificate validation.
        """

        self.alexa = alexa
        self.validate = validate_requests

    @Request.application
    def __call__(self, request):
        return self.wsgi_app(request)

    def wsgi_app(self, request):
        """Validates and routes WSGI requests."""

        try:
            if request.method != 'POST':
                abort(400)

            try:
                body = json.loads(request.data)
            except ValueError:
                abort(400)

            if self.validate:
                valid_cert = util.validate_request_certificate(request)
                valid_ts = util.validate_request_timestamp(body)

                if not valid_cert or not valid_ts:
                    log.error('failed to validate request')
                    abort(403)

            resp_obj = self.alexa.dispatch_request(body)
            return Response(response=json.dumps(resp_obj, indent=4),
                            status=200,
                            mimetype='application/json')

        except HTTPException, exc:
            return exc



from setuptools import setup


setup(
    name='alexandra',
    version='0.0.0',
    description='Toolkit for writing Amazon Alexa skills as web services',
    author='Erik Price',
    url='https://github.com/erik/alexandra',
    packages=['alexandra'],
    license='ISC',
    install_requires=open('requirements.txt').readlines()
)



import json


class Session:
    """Provides easier access to session objects sent along with requests"""

    def __init__(self, session_obj):
        self.obj = session_obj

    def __repr__(self):
        return '<Session %s>' % json.dumps(self.obj)

    @property
    def is_new(self):
        """Is this a new session or a previously open one?"""
        return self.obj['new']

    @property
    def session_id(self):
        return self.obj['sessionId']

    @property
    def application_id(self):
        return self.obj['application']['applicationId']

    @property
    def user_id(self):
        return self.obj['user'].get('userId')

    @property
    def user_access_token(self):
        return self.obj['user'].get('accessToken')

    def get(self, attr, default=None):
        """Get an attribute defined by this session"""

        attrs = self.obj.get('attributes') or {}
        return attrs.get(attr, default)
